#### 目录介绍
- 01.前沿简单介绍
- 02.什么是单元测试
- 03.发现代码中bug
- 04.发现代码设计问题
- 05.对集成测试的有力补充
- 06.有利代码重构的过程
- 07.如何编写单元测试




### 01.前沿简单介绍
- 如何保证重构不出错呢？你需要熟练掌握各种设计原则、思想、模式，还需要对所重构的业务和代码有足够的了解。除了这些个人能力因素之外，最可落地执行、最有效的保证重构不出错的手段应该就是单元测试（Unit Testing）了。当重构完成之后，如果新的代码仍然能通过单元测试，那就说明代码原有逻辑的正确性未被破坏，原有的外部可见行为未变。
- 内容主要包含这样几个内容：什么是单元测试？为什么要写单元测试？如何编写单元测试？如何在团队中推行单元测试？



### 02.什么是单元测试
- 单元测试由研发工程师自己来编写，用来测试自己写的代码的正确性。我们常常将它跟集成测试放到一块来对比。单元测试相对于集成测试（Integration Testing）来说，测试的粒度更小一些。集成测试的测试对象是整个系统或者某个功能模块，比如测试用户注册、登录功能是否正常，是一种端到端（end to end）的测试。而单元测试的测试对象是类或者函数，用来测试一个类和函数是否都按照预期的逻辑执行。这是代码层级的测试。
- 举个例子来解释一下
    ```java
    public class Text {
      private String content;
    
      public Text(String content) {
        this.content = content;
      }
    
      /**
       * 将字符串转化成数字，忽略字符串中的首尾空格；
       * 如果字符串中包含除首尾空格之外的非数字字符，则返回null。
       */
      public Integer toNumber() {
        if (content == null || content.isEmpty()) {
          return null;
        }
        //...省略代码实现...
        return null;
      }
    }
    ```
- 如果我们要测试 Text 类中的 toNumber() 函数的正确性，应该如何编写单元测试呢？
    - 写单元测试本身不需要什么高深技术。它更多的是考验程序员思维的缜密程度，看能否设计出覆盖各种正常及异常情况的测试用例，来保证代码在任何预期或非预期的情况下都能正确运行。
- 为了保证测试的全面性，针对 toNumber() 函数，我们需要设计下面这样几个测试用例。
    - 如果字符串只包含数字：“123”，toNumber() 函数输出对应的整数：123。
    - 如果字符串是空或者 null，toNumber() 函数返回：null。
    - 如果字符串包含首尾空格：“ 123”，“123 ”，“ 123 ”，toNumber() 返回对应的整数：123。
    - 如果字符串包含多个首尾空格：“ 123 ”，toNumber() 返回对应的整数：123；
    - 如果字符串包含非数字字符：“123a4”，“123 4”，toNumber() 返回 null；
- 代码贴在下面了，可以参考一下，这里没有使用任何测试框架
    ```java
    public class Assert {
      public static void assertEquals(Integer expectedValue, Integer actualValue) {
        if (actualValue != expectedValue) {
          String message = String.format(
                  "Test failed, expected: %d, actual: %d.", expectedValue, actualValue);
          System.out.println(message);
        } else {
          System.out.println("Test succeeded.");
        }
      }
    
      public static boolean assertNull(Integer actualValue) {
        boolean isNull = actualValue == null;
        if (isNull) {
          System.out.println("Test succeeded.");
        } else {
          System.out.println("Test failed, the value is not null:" + actualValue);
        }
        return isNull;
      }
    }
    
    public class TestCaseRunner {
      public static void main(String[] args) {
        System.out.println("Run testToNumber()");
        new TextTest().testToNumber();
    
        System.out.println("Run testToNumber_nullorEmpty()");
        new TextTest().testToNumber_nullorEmpty();
    
        System.out.println("Run testToNumber_containsLeadingAndTrailingSpaces()");
        new TextTest().testToNumber_containsLeadingAndTrailingSpaces();
    
        System.out.println("Run testToNumber_containsMultiLeadingAndTrailingSpaces()");
        new TextTest().testToNumber_containsMultiLeadingAndTrailingSpaces();
    
        System.out.println("Run testToNumber_containsInvalidCharaters()");
        new TextTest().testToNumber_containsInvalidCharaters();
      }
    }
    
    public class TextTest {
      public void testToNumber() {
        Text text = new Text("123");
        Assert.assertEquals(123, text.toNumber());
      }
    
      public void testToNumber_nullorEmpty() {
        Text text1 = new Text(null);
        Assert.assertNull(text1.toNumber());
    
        Text text2 = new Text("");
        Assert.assertNull(text2.toNumber());
      }
    
      public void testToNumber_containsLeadingAndTrailingSpaces() {
        Text text1 = new Text(" 123");
        Assert.assertEquals(123, text1.toNumber());
    
        Text text2 = new Text("123 ");
        Assert.assertEquals(123, text2.toNumber());
    
        Text text3 = new Text(" 123 ");
        Assert.assertEquals(123, text3.toNumber());
      }
    
      public void testToNumber_containsMultiLeadingAndTrailingSpaces() {
        Text text1 = new Text("  123");
        Assert.assertEquals(123, text1.toNumber());
    
        Text text2 = new Text("123  ");
        Assert.assertEquals(123, text2.toNumber());
    
        Text text3 = new Text("  123  ");
        Assert.assertEquals(123, text3.toNumber());
      }
    
      public void testToNumber_containsInvalidCharaters() {
        Text text1 = new Text("123a4");
        Assert.assertNull(text1.toNumber());
    
        Text text2 = new Text("123 4");
        Assert.assertNull(text2.toNumber());
      }
    }
    ```


### 03.发现代码中bug
- 能否写出 bug free 的代码，是判断工程师编码能力的重要标准之一，也是很多大厂面试考察的重点，特别是像 FLAG 这样的外企。可以这么说，坚持写单元测试是保证我的代码质量的一个“杀手锏”。



### 04.发现代码设计问题
- 代码的可测试性是评判代码质量的一个重要标准。对于一段代码，如果很难为其编写单元测试，或者单元测试写起来很吃力，需要依靠单元测试框架里很高级的特性才能完成，那往往就意味着代码设计得不够合理，比如，没有使用依赖注入、大量使用静态函数、全局变量、代码高度耦合等。



### 05.对集成测试的有力补充
- 程序运行的 bug 往往出现在一些边界条件、异常情况下，比如，除数未判空、网络超时。而大部分异常情况都比较难在测试环境中模拟。而单元测试可以利用下一节课中讲到的 mock 的方式，控制 mock 的对象返回我们需要模拟的异常，来测试代码在这些异常情况的表现。
- 除此之外，对于一些复杂系统来说，集成测试也无法覆盖得很全面。复杂系统往往有很多模块。每个模块都有各种输入、输出、异常情况，组合起来，整个系统就有无数测试场景需要模拟，无数的测试用例需要设计，再强大的测试团队也无法穷举完备。
- 尽管单元测试无法完全替代集成测试，但如果我们能保证每个类、每个函数都能按照我们的预期来执行，底层 bug 少了，那组装起来的整个系统，出问题的概率也就相应减少了。


### 06.有利代码重构的过程
- 要把持续重构作为开发的一部分来执行，那写单元测试实际上就是落地执行持续重构的一个有效途径。设计和实现代码的时候，我们很难把所有的问题都想清楚。而编写单元测试就相当于对代码的一次自我 Code Review，在这个过程中，我们可以发现一些设计上的问题（比如代码设计的不可测试）以及代码编写方面的问题（比如一些边界条件处理不当）等，然后针对性的进行重构。


### 07.如何编写单元测试
- 举了一个给 toNumber() 函数写单元测试的例子。根据那个例子，我们可以总结得出，写单元测试就是针对代码设计覆盖各种输入、异常、边界条件的测试用例，并将这些测试用例翻译成代码的过程。
- 





